// lex.h
// VERSION 2
// Glenn G. Chappell
// 4 Feb 2013
//
// For CS 331 Spring 2013
// Header for class Lex
// State-Machine-Based Lexical Analyzer

#ifndef FILE_LEX_H_INCLUDED
#define FILE_LEX_H_INCLUDED

#include <string>   // for std::string
#include <utility>  // for std::pair
#include <cstdlib>  // for std::size_t


// class Lex
// Lexical Analyzer
// Usable in the following manner (theprogram is the string to be lexed):
//
//     Lex luthor;
//     for (luthor.set(theprogram); !luthor.done(); luthor.advance())
//     {
//         Lex::Lexeme myword = luthor.current();
//         // Use myword here
//     }
class Lex {
// ***** Lex: Types *****
public:

    // Type for token (category) of lexeme
    // Token NONE is for lexemes returned when we
    //  are past the last lexeme in the input.
    enum Token { NONE, ILLEGAL, KEY, ID, OP, PUNCT, NUM };

    // Type for lexemes returned to client code
    typedef std::pair<std::string, Token> Lexeme;

// ***** Lex: ctors, dctor, op= *****
public:

    // Ctor from string & default ctor
    // Sets input to the given string.
    // Make currlexeme_ the first lexeme, or ("", NONE) if there
    //  are no lexemes.
    Lex(const std::string & input="")
    { set(input); }

    // Compiler-generated copy ctor, copy =, dctor used

// ***** Lex: general public functions *****
public:

    // set
    // Sets input to the given string.
    // Make currlexeme_ the first lexeme, or ("", NONE) if there
    //  are no lexemes.
    void set(const std::string & input)
    {
        input_ = input;
        pos_ = 0;
        skipSpace();
        advance();
    }

    // done
    // Returns true if we are past the last lexeme in the input.
    bool done() const
    { return currlexeme_.second == NONE; }

    // advance
    // Moves to the next lexeme -- or ("", NONE) if there are no
    //  more -- and sets currlexeme_ equal to it.
    void advance();

    // current
    // Return the current lexeme (currlexeme_).
    Lexeme current() const
    { return currlexeme_; }

// ***** Lex: internal-use functions *****
private:

    // skipSpace
    // Sets pos_ to the index of the start of the next lexeme after its
    //  initial location, or input_.size() if there are no further
    //  lexemes.
    void skipSpace();

// ***** Lex: data members *****
private:

    std::string input_;  // The input to be lexed
    Lexeme currlexeme_;  // Current lexeme, returned to client code
    std::size_t pos_;    // Index in input_ of the start of the next
                         //  lexeme, or input_.size() if there are no
                         //  others.

};  // End class lex

#endif //#ifndef FILE_LEX_H_INCLUDED

